apiVersion: v1
kind: Service
metadata:
  name: {{ service_name }}
  labels:
  {% for key, value in labels.items() %}
    {{ key }}: "{{ value }}"
  {% endfor %}
spec:
  type: NodePort
  selector:
  {% for key, value in selector.items() %}
    {{ key}}: "{{ value }}"
  {% endfor %}
  ports:
    - protocol: TCP
      port: 9090
      targetPort: 9090
