# Copyright 2018 OpenStack Foundation.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

"""add cpu column for functions

Revision ID: 003
Revises: 002
Create Date: 2018-04-19 06:03:46.687706

"""

# revision identifiers, used by Alembic.
revision = '003'
down_revision = '002'

from alembic import op
import sqlalchemy as sa


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column(
        'functions',
        sa.Column('cpu', sa.Integer(), nullable=False)
    )

    op.alter_column(
        'functions',
        'memory_size',
        existing_type=sa.Integer(),
        nullable=False
    )
    # ### end Alembic commands ###
