# Copyright 2017 Catalyst IT Limited
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.
CURRENT_VERSION = 'v1'

EXECUTION_BY_JOB = 'Created by Job %s'
EXECUTION_BY_WEBHOOK = 'Created by Webhook %s'

PERIODIC_JOB_HANDLER = 'job_handler'
PERIODIC_FUNC_MAPPING_HANDLER = 'function_mapping_handler'

PACKAGE_FUNCTION = 'package'
SWIFT_FUNCTION = 'swift'
IMAGE_FUNCTION = 'image'

MAX_PACKAGE_SIZE = 51 * 1024 * 1024

MAX_VERSION_NUMBER = 10
