============
API Versions
============

Show all enabled API versions

List all API versions
=====================

.. rest_method::  GET /

.. rest_status_code:: success status.yaml

   - 200

.. rest_status_code:: error status.yaml

   - 400
   - 401
   - 403

Request
-------

No parameters needed

Response Example
----------------

.. literalinclude:: samples/api-versions/get-versions-response.json
   :language: javascript
