apiVersion: v1
kind: Pod
metadata:
  name: {{ pod_name }}
  labels:
  {% for key, value in labels.items() %}
    {{ key }}: {{ value }}
  {% endfor %}
  annotations:
    io.kubernetes.cri-o.TrustedSandbox: "false"
spec:
  terminationGracePeriodSeconds: 5
  automountServiceAccountToken: false
  containers:
    - name: {{ pod_name }}
      image: {{ pod_image }}
      imagePullPolicy: IfNotPresent
      {% if input %}
      args:
      {% for item in input %}
        - "{{ item | safe }}"
      {% endfor %}
      {% endif %}
  restartPolicy: Never
  resources:
      limits:
        cpu: {{ limit_cpu }}m
        memory: {{ limit_memory }}
      requests:
        cpu: {{ req_cpu }}m
        memory: {{ req_memory }}
